local _G = _G
local Cartographer3 = _G.Cartographer3
local L = Cartographer3.L

local panel = CreateFrame("Frame", "Cartographer3_Panel")

panel.name = "Cartographer3"
panel.controls = {}

function panel:okay()
	for control in pairs(self.controls) do
		control.value = control.oldValue
	end
end

function panel:cancel()
	for control in pairs(self.controls) do
		control:SetValue(control.oldValue)
	end
end

function panel:default()
	for control in pairs(self.controls) do
		control:SetValue(control.default)
	end
end

InterfaceOptions_AddCategory(panel)

local makeControls
panel:SetScript("OnShow", function(self)
	self:SetScript("OnShow", nil)
	makeControls(self)
	makeControls = nil
end)

function makeControls(self)
	local title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
	title:SetText("Cartographer3")
	title:SetJustifyH("LEFT")
	title:SetJustifyV("TOP")
	title:SetPoint("TOPLEFT", 16, -16)
	
	local subText = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	subText:SetText(L["These options allow you to configure Cartographer3"]) -- TODO: better text
	subText:SetNonSpaceWrap(true)
	subText:SetJustifyH("LEFT")
	subText:SetJustifyV("TOP")
	subText:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -8)
	subText:SetPoint("RIGHT", -32, 0)
	
	local versionText = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	local format, height, website
	if Cartographer3.versionType == "Development" then
		format = "You are running %s from the source code repository. This is for |cffff7f7fdevelopers only|r. It is recommended in nearly all cases to instead use an alpha version from %s" -- no need for localization
		height = 40
		website = "http://wow.curseforge.com/projects/cartographer3/"
	elseif Cartographer3.versionType == "Alpha" then
		format = L["Version %s: This is for alpha testers wanting to try out new features and have possibly bad bugfixes |cffff7f7fbefore they are ready|r. It is recommended to instead use a beta or release version from %s"]
		height = 40
		website = "curse.com"
	elseif Cartographer3.versionType == "Beta" then
		format = L["Version %s: This is a beta version for those who want to test new features and bugfixes before they are potentially ready. It is recommended to use a release version from %s"]
		height = 40
		website = "curse.com"
	elseif Cartographer3.versionType == "Release" then
		format = L["Version %s. There may be more up-to-date versions on %s"]
		height = 20
		website = "curse.com"
	end
	versionText:SetText(format:format(Cartographer3.version, website))
	versionText:SetNonSpaceWrap(true)
	versionText:SetJustifyH("LEFT")
	versionText:SetJustifyV("TOP")
	versionText:SetPoint("TOPLEFT", subText, "BOTTOMLEFT", 0, -8)
	versionText:SetPoint("RIGHT", -32, 0)
	versionText:SetHeight(height)
	
	local function makeZoomToMinimapTextureSlider()
		local zoomToMinimapTextureSlider = CreateFrame("Slider", panel:GetName() .. "_ZoomToMinimapTextureSlider", panel, "InterfaceOptionsSliderTemplate")
		self.controls[zoomToMinimapTextureSlider] = true
		_G[zoomToMinimapTextureSlider:GetName() .. "Text"]:SetText(L["Texture Change Zoom"])
		zoomToMinimapTextureSlider.tooltipText = L["The amount to zoom in before the map changes from minimap-style textures to artwork.\n\nNote: Setting this too far may cause severe framerate issues."]
		_G[zoomToMinimapTextureSlider:GetName() .. "Text"]:SetTextColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		_G[zoomToMinimapTextureSlider:GetName() .. "Low"]:SetText(L["Far"])
		_G[zoomToMinimapTextureSlider:GetName() .. "Low"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		_G[zoomToMinimapTextureSlider:GetName() .. "High"]:SetText(L["Near"])
		_G[zoomToMinimapTextureSlider:GetName() .. "High"]:SetTextColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
		zoomToMinimapTextureSlider.default = Cartographer3.Data.DEFAULT_ZOOM_TO_MINIMAP_TEXTURE
		zoomToMinimapTextureSlider:SetMinMaxValues(0, 120)
		zoomToMinimapTextureSlider.oldValue = Cartographer3.db.zoomToMinimapTexture
		zoomToMinimapTextureSlider:SetValue(zoomToMinimapTextureSlider.oldValue)
		local oldSetValue = zoomToMinimapTextureSlider.SetValue
		function zoomToMinimapTextureSlider:SetValue(value)
			oldSetValue(self, value)
			self.value = value
			Cartographer3.db.zoomToMinimapTexture = value
		end
		zoomToMinimapTextureSlider:SetScript("OnValueChanged", function(self)
			self.value = self:GetValue()
			self:SetValue(self:GetValue())
		end)
		return zoomToMinimapTextureSlider
	end
	
	local function makeStrataDropDown()
		local strataDropDown = CreateFrame("Frame", panel:GetName() .. "_StrataDropDown", panel, "UIDropDownMenuTemplate")
		self.controls[strataDropDown] = true
		local label = strataDropDown:CreateFontString(nil, "BACKGROUND", "GameFontNormal")
		label:SetText(L["Strata"])
		strataDropDown.tooltipText = L["What level on the interface the map will show up on. 'Fullscreen' is the highest possible level and 'Background' is the lowest."]
		label:SetPoint("BOTTOMLEFT", strataDropDown, "TOPLEFT", 16, 3)
		local stratas = {
			"FULLSCREEN", L["Fullscreen"],
			"DIALOG", L["Dialog"],
			"HIGH", L["High"],
			"MEDIUM", L["Medium"],
			"LOW", L["Low"],
			"BACKGROUND", L["Background"]
		}
		UIDropDownMenu_Initialize(strataDropDown, function()
			for i = 1, #stratas, 2 do
				local value = stratas[i]
				local text = stratas[i+1]

				local info = UIDropDownMenu_CreateInfo()
				info.text = text
				info.value = value
				info.checked = Cartographer3.db.strata == value
				info.func = strataDropDown.SetValue
				info.arg1 = strataDropDown
				info.arg2 = value
				UIDropDownMenu_AddButton(info)
			end
		end)
		UIDropDownMenu_SetWidth(90, strataDropDown)
		UIDropDownMenu_SetSelectedValue(strataDropDown, Cartographer3.db.strata)
		strataDropDown.default = "FULLSCREEN"
		strataDropDown.oldValue = Cartographer3.db.strata
		function strataDropDown:SetValue(value)
			self.value = value
			Cartographer3.SetFrameStrata(value)
			UIDropDownMenu_SetSelectedValue(strataDropDown, value)
		end
		strataDropDown:EnableMouse(true)
		strataDropDown:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
			GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, 1)
		end)
		strataDropDown:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
		return strataDropDown
	end
	
	local url
	local function makeURLFrame()
		makeURLFrame = nil
		local function bumpFrameLevels(frame, amount)
			frame:SetFrameLevel(frame:GetFrameLevel()+amount)
			local children = newList(frame:GetChildren())
			for _,v in ipairs(children) do
				bumpFrameLevels(v, amount)
			end
			children = del(children)
		end
		-- some code borrowed from Prat here
		StaticPopupDialogs["ROCK_SHOW_URL"] = {
			text = not IsMacClient() and L["Press Ctrl-C to copy, then Alt-Tab out of the game, open your favorite web browser, and paste the link into the address bar."] or L["Press Cmd-C to copy, then Cmd-Tab out of the game, open your favorite web browser, and paste the link into the address bar."],
			button2 = ACCEPT,
			hasEditBox = 1,
			hasWideEditBox = 1,
			showAlert = 1, -- HACK : it's the only way I found to make de StaticPopup have sufficient width to show WideEditBox :(

			OnShow = function()
				local editBox = _G[this:GetName() .. "WideEditBox"]
				editBox:SetText(url)
				editBox:SetFocus()
				editBox:HighlightText(0)
				editBox:SetScript("OnTextChanged", function() StaticPopup_EditBoxOnTextChanged() end)

				local button = _G[this:GetName() .. "Button2"]
				button:ClearAllPoints()
				button:SetWidth(200)
				button:SetPoint("CENTER", editBox, "CENTER", 0, -30)

				_G[this:GetName() .. "AlertIcon"]:Hide()  -- HACK : we hide the false AlertIcon
				this:SetFrameStrata("FULLSCREEN_DIALOG")
				bumpFrameLevels(this, 30)
			end,
			OnHide = function()
				local editBox = _G[this:GetName() .. "WideEditBox"]
				editBox:SetScript("OnTextChanged", nil)
				this:SetFrameStrata("DIALOG")
				bumpFrameLevels(this, -30)
			end,
			OnAccept = function() end,
			OnCancel = function() end,
			EditBoxOnEscapePressed = function() this:GetParent():Hide() end,
			EditBoxOnTextChanged = function()
				this:SetText(url)
				this:SetFocus()
				this:HighlightText(0)
			end,
			timeout = 0,
			whileDead = 1,
			hideOnEscape = 1
		}
	end

	function OpenDonationFrame(self)
		if makeURLFrame then
			makeURLFrame()
		end

		local donate = self.donate
		if type(donate) ~= "string" then
			donate = "Wowace"
		end
		local style, data = (":"):split(donate, 2)
		style = style:lower()
		if style ~= "website" and style ~= "paypal" then
			style = "wowace"
		end
		if style == "wowace" then
			url = "http://www.wowace.com/wiki/Donations"
		elseif style == "website" then
			url = data
		else -- PayPal
			local text = "https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=" .. urlencode(unobfuscateEmail(data))
			local name
			if type(self.title) == "string" then
				name = self.title
			elseif type(self.name) == "string" then
				name = self.name
			end
			if name == MAJOR_VERSION then
				name = "Rock"
			end
			if name then
				name = name:gsub("|c%x%x%x%x%x%x%x%x", ""):gsub("|r", "")
				text = text .. "&item_name=" .. urlencode(name)
			end
			url = text
		end

		StaticPopup_Show("ROCK_SHOW_URL")
	end
	local openReportIssueBox, openDonateBox
	do
		local url
		local function makeURLFrame()
			makeURLFrame = nil
			local function bumpFrameLevels(frame, amount)
				frame:SetFrameLevel(frame:GetFrameLevel()+amount)
				local children = newList(frame:GetChildren())
				for _,v in ipairs(children) do
					bumpFrameLevels(v, amount)
				end
				children = del(children)
			end
			-- some code borrowed from Prat here
			StaticPopupDialogs["CARTOGRAPHER3_SHOW_URL"] = {
				text = not IsMacClient() and L["Press Ctrl-C to copy, then Alt-Tab out of the game, open your favorite web browser, and paste the link into the address bar."] or L["Press Cmd-C to copy, then Cmd-Tab out of the game, open your favorite web browser, and paste the link into the address bar."],
				button2 = ACCEPT,
				hasEditBox = 1,
				hasWideEditBox = 1,
				showAlert = 1, -- HACK : it's the only way I found to make de StaticPopup have sufficient width to show WideEditBox :(

				OnShow = function()
					local editBox = _G[this:GetName() .. "WideEditBox"]
					editBox:SetText(url)
					editBox:SetFocus()
					editBox:HighlightText(0)
					editBox:SetScript("OnTextChanged", function() StaticPopup_EditBoxOnTextChanged() end)

					local button = _G[this:GetName() .. "Button2"]
					button:ClearAllPoints()
					button:SetWidth(200)
					button:SetPoint("CENTER", editBox, "CENTER", 0, -30)

					_G[this:GetName() .. "AlertIcon"]:Hide()  -- HACK : we hide the false AlertIcon
					this:SetFrameStrata("FULLSCREEN_DIALOG")
					this:SetFrameLevel(this:GetFrameLevel()+30)
				end,
				OnHide = function()
					local editBox = _G[this:GetName() .. "WideEditBox"]
					editBox:SetScript("OnTextChanged", nil)
					this:SetFrameStrata("DIALOG")
					this:SetFrameLevel(this:GetFrameLevel()-30)
				end,
				OnAccept = function() end,
				OnCancel = function() end,
				EditBoxOnEscapePressed = function() this:GetParent():Hide() end,
				EditBoxOnTextChanged = function()
					this:SetText(url)
					this:SetFocus()
					this:HighlightText(0)
				end,
				timeout = 0,
				whileDead = 1,
				hideOnEscape = 1
			}
		end
		
		function openReportIssueBox()
			if makeURLFrame then
				makeURLFrame()
			end
		
			url = "http://wow.curseforge.com/projects/cartographer3/tickets/"

			StaticPopup_Show("CARTOGRAPHER3_SHOW_URL")
		end
		
		function openDonateBox()
			if makeURLFrame then
				makeURLFrame()
			end
			
			url = "https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=ckknight%40gmail%2ecom&item_name=Cartographer3"

			StaticPopup_Show("CARTOGRAPHER3_SHOW_URL")
		end
	end
	
	local function makeReportButton()
		local reportButton = CreateFrame("Button", panel:GetName() .. "_ReportButton", panel, "UIPanelButtonTemplate2")
		self.controls[reportButton] = true
		reportButton:SetText(L["Report Issue"])
		reportButton.tooltipText = L["Report a defect or a possible enhancement you see with Cartographer3."]
		reportButton:SetWidth(120)
		reportButton:SetHeight(22)
		function reportButton:SetValue()
		end
		reportButton:SetScript("OnClick", function()
			openReportIssueBox()
		end)
		reportButton:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, 1)
		end)
		reportButton:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
		return reportButton
	end
	
	local function makeDonateButton()
		local donateButton = CreateFrame("Button", panel:GetName() .. "_DonateButton", panel, "UIPanelButtonTemplate2")
		self.controls[donateButton] = true
		donateButton:SetText(L["Donate"])
		donateButton.tooltipText = L["Give a much-needed donation to ckknight, the author of Cartographer3"]
		donateButton:SetWidth(120)
		donateButton:SetHeight(22)
		function donateButton:SetValue()
		end
		donateButton:SetScript("OnClick", function()
			openDonateBox()
		end)
		donateButton:SetScript("OnEnter", function(self)
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetText(self.tooltipText, nil, nil, nil, nil, 1)
		end)
		donateButton:SetScript("OnLeave", function(self)
			GameTooltip:Hide()
		end)
		return donateButton
	end
	
	local reportButton = makeReportButton()
	reportButton:SetPoint("TOPLEFT", versionText, "BOTTOMLEFT", 0, -16)
	
	local donateButton = makeDonateButton()
	donateButton:SetPoint("TOPLEFT", reportButton, "BOTTOMLEFT", 0, -16)
	
	local zoomToMinimapTextureSlider = makeZoomToMinimapTextureSlider()
	zoomToMinimapTextureSlider:SetPoint("TOPLEFT", donateButton, "BOTTOMLEFT", 0, -32)
	
	local strataDropDown = makeStrataDropDown()
	strataDropDown:SetPoint("TOPLEFT", zoomToMinimapTextureSlider, "BOTTOMLEFT", -13, -28)
end
